# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Grain'
        db.create_table(u'grains_grain', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('key', self.gf('django.db.models.fields.CharField')(unique=True, max_length=1024, db_index=True)),
            ('content_type', self.gf('django.db.models.fields.CharField')(default='text/plain', max_length=255)),
            ('value', self.gf('django.db.models.fields.TextField')(blank=True)),
        ))
        db.send_create_signal(u'grains', ['Grain'])


    def backwards(self, orm):
        # Deleting model 'Grain'
        db.delete_table(u'grains_grain')


    models = {
        u'grains.grain': {
            'Meta': {'object_name': 'Grain'},
            'content_type': ('django.db.models.fields.CharField', [], {'default': "'text/plain'", 'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '1024', 'db_index': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['grains']


from django.template import Library, Node, TemplateSyntaxError, Variable
from ..models import Grain

register = Library()


class GrainNode(Node):
    def __init__(self, nodelist, key, content_type):
        self.nodelist = nodelist
        self.key = Variable(key)
        self.content_type = Variable(content_type)

    def render(self, context):
        key = self.key.resolve(context)
        content_type = self.content_type.resolve(context)
        default_content = self.nodelist.render(context)
        grain_instance, created = Grain.objects.get_or_create(
            key=key, defaults={'content_type': content_type,
                               'value': default_content})
        if not created and grain_instance.content_type != content_type:
            grain_instance.content_type = content_type
            grain_instance.save()
        return grain_instance.value


@register.tag
def grain(parser, token):
    # pylint: disable=W0613
    #         Unused argument

    bits = token.split_contents()
    if len(bits) not in (2, 3):
        raise TemplateSyntaxError(
            "%r tag requires one or two arguments" % bits[0])
    nodelist = parser.parse(('endgrain',))
    parser.delete_first_token()
    key = bits[1]
    content_type = bits[2] if len(bits) >= 3 else '"text/plain"'
    return GrainNode(nodelist, key, content_type)



from django.db import models


class Grain(models.Model):
    key = models.CharField(max_length=1024, unique=True, db_index=True)
    content_type = models.CharField(max_length=255, default='text/plain')
    value = models.TextField(blank=True)



from django.conf import settings
from django.contrib import admin
from .models import Grain


class GrainAdmin(admin.ModelAdmin):
    model = Grain
    list_display = 'key', 'value'
    fields = 'value',

    def render_change_form(self, request, context,
                           add=False, change=False, form_url='', obj=None):
        if (obj.content_type == u'text/html'
             and 'django_wysiwyg' in settings.INSTALLED_APPS):
            self.change_form_template = 'grains/admin/wysiwyg_change_form.html'
        return super(GrainAdmin, self).render_change_form(
            request, context,
            add=add, change=change, form_url=form_url, obj=obj)


admin.site.register(Grain, GrainAdmin)



from setuptools import setup


setup(name='django-grains',
      version='0.2',
      description=('Define content fragments in Django templates '
                   'and store their contents in the database. '
                   'This is yet another take '
                   'on the idea previously implemented in '
                   'django-flatblocks, django-chunks etc.'),
      author='Antti Kaihola (2General Ltd.)',
      author_email='antti@2general.com',
      url='http://github.com/2general/django-grains',
      packages=['grains', 'grains.migrations', 'grains.templatetags'],
      classifiers=['Development Status :: 4 - Beta',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Internet :: WWW/HTTP',
                   'License :: OSI Approved :: BSD License'])
